#include <stdio.h>
#include <string.h>

#include <fmt.h>
#include <rss-handler.h>
#include <xml-parser.h>
#include <xml-writer.h>

int rss_err (char *point)
{
	fprintf(stderr, "RSS: Error in %s.\n", point);
	return 1;
}

bool rss_init (feed *fd, char *furl)
{
	return true;
}

char *rss_searchval (xmlNodePtr root, char *sstr)
{
	char *t1, *t2, *t3;

	t3=xml_searchval(root, sstr);
	t1=fmt_html(t3);
	free(t3);
	t2=t1;
	t1=fmt_utf8(t1);
	free(t2);
	t2=t1;
	t1=fmt_rmesc(t1);
	free(t2);

	return t1;
}

bool rss_init_cache (feed *fd, char *cfile)
{
	char *tmp;

	if (!fd)
		return rss_err("feed allocation");

	fd->doc=xmlParseFile(cfile);
	if (!fd->doc)
		return !rss_err("xmlParseFile");
		
	fd->root=xmlDocGetRootElement(fd->doc);
	
	strncpy(fd->furl, "", RF_LEN);
	strncpy(fd->cfile, cfile, RF_LEN);
	fd->cur=NULL;

	/* RSS 1.0 */
	if (!strcmp((char *)fd->root->name, "RDF"))
		fd->V=RSS_10;

	if (!strcmp((char *)fd->root->name, "rss"))
	{
		/* RSS 2.0 */
		tmp=xml_prop(fd->root, "version");
		if (!strcmp(tmp, "2.0"))
			fd->V=RSS_20;
		free(tmp);
		
		/* RSS 0.92 */
		tmp=xml_prop(fd->root, "version");
		if (!strcmp(tmp, "0.92"))
			fd->V=RSS_092;
		free(tmp);
		
		/* RSS 0.91 */
		tmp=xml_prop(fd->root, "version");
		if (!strcmp(tmp, "0.91"))
			fd->V=RSS_091;
		free(tmp);
		
		/* RSS 0.94 */
		tmp=xml_prop(fd->root, "version");
		if (!strcmp(tmp, "0.94"))
			fd->V=RSS_094;
		free(tmp);
	}

	/* Atom 0.3 */
	tmp=xml_prop(fd->root, "version");
	if ((!strcmp((char *)fd->root->name, "feed"))&&(!strcmp(tmp, "0.3")))
		fd->V=ATOM_03;
	free(tmp);

	rss_reset(fd);
	
	tmp=rss_searchval(fd->cur, "title");
	strncpy(fd->title, tmp, RF_LEN);
	free(tmp);

	tmp=rss_searchval(fd->cur, "link");
	strncpy(fd->url, tmp, RF_LEN);
	free(tmp);
	
	tmp=rss_searchval(fd->cur, "description");
	strncpy(fd->desc, tmp, RF_DESCLEN);
	free(tmp);

	return true;
}

char *rss_spam_filter (char *item)
{
	char *t1=(char *)malloc(strlen(item)+strlen(SPAMMSG)+2);
	
	strcpy(t1, item);
	
	if (!strcmp(item, "Customize this feed"))
		sprintf(t1, "%s %s", SPAMMSG, item);
	if (!strcmp(item, "Lookup"))
		sprintf(t1, "%s %s", SPAMMSG, item);

	return t1;
}

bool rss_next (feed *fd, newsitem *item)
{	
	char *tmp;

	if (!item)
	{
		rss_err("rss_next (item==NULL)");
		return false;
	}

	strcpy(item->title, "");
	strcpy(item->url, "");
	strcpy(item->desc, "");
	
	if ((!fd->root)||(!fd->cur)||(!fd->cur->children))
		return false;

	tmp=rss_searchval(fd->cur->children, "title");
	strncpy(item->title, tmp, RF_LEN);
	free(tmp);

	item->read=rss_readstat(fd->cur->children);

	switch (fd->V)
	{
		case ATOM_03:
			tmp=xml_prop(xml_searchptr(fd->cur->children, "link"), "href");
			strncpy(item->url, tmp, RF_LEN);
			free(tmp);
			
			tmp=rss_searchval(fd->cur->children, "summary");
			strncpy(item->desc, tmp, RF_DESCLEN);
			free(tmp);
			break;
		default:
			tmp=xml_val(xml_searchptr(fd->cur->children, "link"));
			strncpy(item->url, tmp, RF_LEN);
			free(tmp);
			
			tmp=rss_searchval(fd->cur->children, "description");
			strncpy(item->desc, tmp, RF_DESCLEN);
			free(tmp);
			break;
	}

	fd->cur=rss_nextptr(fd);
	
	tmp=rss_spam_filter(item->title);
	strcpy(item->title, tmp);
	free(tmp);

	return true;
}

bool rss_readstat (xmlNodePtr node)
{
	char *str=rss_searchval(node, "readstatus");

	if (!strcmp(str, ""))
	{
		xml_new_node(node->parent, "readstatus", "0\n");
		free(str);
		return false;
	}

	if (!strcmp(str, "0"))
	{
		free(str);
		return false;
	}

	free(str);
	return true;
}

bool rss_begin (feed *fd)
{
	rss_reset(fd);
	
	switch (fd->V) 
	{
		case RSS_10:
			fd->cur=fd->cur->parent->next->next;
			break;
		case ATOM_03:
			fd->cur=xml_searchptr(fd->root->children, "entry");
			break;
		default:
			fd->cur=xml_searchptr(fd->cur, "item");
			break;
	}

	return true;
}

bool rss_reset (feed *fd)
{
	switch (fd->V)
	{
		case RSS_10:
			fd->cur=rss10_chan(fd->root);
			break;
		case RSS_094:
			fd->cur=rss094_chan(fd->root);
			break;
		case ATOM_03:
			fd->cur=fd->root->children;
			break;
		default:
			fd->cur=rss20_chan(fd->root);
			break;
	}
	return true;
}

bool rss_end (feed *fd)
{
	/*xmlSaveFile(fd->cfile, fd->doc);*/
	
	if (fd->doc)
		xmlFreeDoc(fd->doc);
	
	return true;
}

bool rss_dump (FILE *file, newsitem item)
{
	fprintf (file, "%s - %s - %s - %i\n", item.title, item.url, item.desc, item.read);	
	return true;
}

bool rss_dump_feed (FILE *file, feed *fd)
{
	int i;
	newsitem *items;

	items=rss_parse(fd, false);
	for (i=0;i<fd->count;i++)
		rss_dump(stdout, items[i]);
	
	return true;
}

xmlNodePtr rss_nextptr (feed *fd)
{
	if (fd->V==RSS_094)
		return fd->cur->next;
	else
		return fd->cur->next->next;
}

int rss_count (feed *fd, bool nospam)
{
	char *tmp, *tmp2;
	int i=0;
	xmlNodePtr start=fd->cur;
	
	while (fd->cur)
	{
		fd->cur=rss_nextptr(fd);
		if (fd->cur)
		{
			tmp2=xml_searchval(fd->cur->children, "title");
			tmp=rss_spam_filter(tmp2);
			if (strncmp(tmp, SPAMMSG, strlen(SPAMMSG)))	
				i++;
			free(tmp);
			free(tmp2);
		}
	}
	
	fd->cur=start;

	return i;
}

newsitem *rss_parse (feed *fd, bool nospam)
{
	int i;
	newsitem *ret;
	
	rss_begin(fd);
	fd->count=1+rss_count(fd, nospam);
	ret=(newsitem *)malloc(fd->count*sizeof(newsitem));
	
	i=0;
	while (fd->cur)
	{
		rss_next(fd, &ret[i]);
		if ((strncmp(ret[i].title, SPAMMSG, strlen(SPAMMSG)))||!nospam)
			i++;
	}

	return ret;
}

void rss_unparse (feed *fd, newsitem *ret)
{
	free(ret);	
}

xmlNodePtr rss094_chan(xmlNodePtr node)
{
	return node->children->children;
}

xmlNodePtr rss10_chan(xmlNodePtr node)
{
	xmlNodePtr ret=xml_searchptr(node->children, "channel");
	return ret->children;
}

xmlNodePtr rss20_chan(xmlNodePtr node)
{
	return node->children->next->children;
}
