#include <stdbool.h>
#include <string.h>

#include <xml-parser.h>

int xml_err (char *point)
{
	fprintf(stderr, "XML: Error in %s.\n", point);
	return 1;
}

char *xml_name (xmlNodePtr node)
{
	return (char *)node->name;
}

char *xml_val (xmlNodePtr node)
{
	return (char *)xmlNodeGetContent(node);
}

char *xml_prop (xmlNodePtr node, char *property)
{
	return (char *)xmlGetProp(node, (xmlChar *)property);
}

void xml_print (FILE *file, xmlNodePtr node)
{
	char *tmp;
	
	if (!node)
	{
		xml_err("xml_print");
		return;
	}

	tmp=xml_val(node);
	fprintf(file, "%s: %s\n", xml_name(node), tmp);
	free(tmp);
}

xmlNodePtr xml_searchptr (xmlNodePtr root, char *sstr)
{
	xmlNodePtr node=root;
	bool found=false;

	if (!node)
	{
		xml_err("xml_searchptr (NULL pointer)");
		return NULL;
	}

	if (!strcmp((char *)node->name, sstr))
		return node;

	while (node->next)
	{
		if (!strcmp((char *)node->name, sstr))
		{
			found=true;
			break;
		}
		node=node->next;
	}

	if (!found)
		return root;
		
	return node;
}

char *xml_searchval (xmlNodePtr root, char *sstr)
{
	xmlNodePtr ret=xml_searchptr(root, sstr);

	if ((ret==root)||(!ret))
	{
		char *ret=(char *)malloc(2);
		strcpy(ret, "");
		return ret;
	}
	
	return xml_val(ret);
}

void xml_dump_tree (FILE *file, xmlNodePtr root)
{
	xmlNodePtr cur=root->children;

	if (cur)
	{
		while (cur)
		{
			xml_print(file, cur);
			cur=cur->next;
		}
	}
}

xmlNodePtr xml_last (xmlNodePtr node)
{
	xmlNodePtr ret;

	ret=node;
	while (ret->next)
	{
		ret=ret->next;
	}

	return ret;
}
