#ifndef __SOCK_H_
#define __SOCK_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <arpa/inet.h>

#define SOCK_UN		1
#define SOCK_IN		2

typedef struct
{
	int type;
	int fd;
	struct sockaddr *adr;
	struct sockaddr_un un_adr;
	struct sockaddr_in in_adr;
	int len;
} sock;

/** Opens a socket
@param sck The socket to work with
@param ip Name or IP address to use
@param port The network port to use (set to -1 if using SOCK_UN)
*/
void sock_open (sock *sck, char *ip, int port);
/** Connect to a socket
@param sck The socket to work with
@return 1 on error, 0 on success
*/
int sock_con (sock *sck);
/** Send a string to a socket
@param sck The socket to work with
@param data The payload
@return Return value of the write()-call
*/
int sock_write (sock *sck, char *data);
/** Receive a string on a socket
@param sck The socket to work with
@param count Number of characters to read 
@return The received data
Return value must be free'd.
*/
char *sock_read (sock *sck, int count);
/** Close a socket
@param sck The socket to work with
@param freeme When this is true, the socket pointer's memory will be free'd
@return 1 on error, 0 on success
*/
int sock_close (sock *sck, bool freeme);
/** Bind to a socket
@param sck The socket to work with
@return 1 on error, 0 on success
*/
int sock_bind (sock *sck);
/** Create a listen queue
@param sck The socket to work with
@param cons Number of connections
@return 1 on error, 0 on success
*/
int sock_listen (sock *sck, int cons);
/** Accept connections to a socket
@param lsock The socket which accepts connections
@param sck The socket of the connecting client
@return 1 on error, 0 on success
*/
int sock_accept (sock *lsock, sock *sck);
/** Receive one character from a socket
@param sck The socket to work with
@return The received character
*/
char sock_readch (sock *sck);
/** Send one character to a socket
@param sck The socket to work with
@param ch The character to send
@return Return value of the sock_write()-call
*/
int sock_writech (sock *sck, char ch);

#ifdef __cplusplus
}
#endif
#endif
